=== modified file 'breezy/git/branch.py'
--- breezy/git/branch.py	2018-11-16 10:50:21 +0000
+++ breezy/git/branch.py	2018-11-16 11:31:40 +0000
@@ -134,10 +134,14 @@
                 pass
             elif overwrite or not ref_name in target_repo._git.refs:
                 target_repo._git.refs[ref_name] = unpeeled or peeled
-                updates[tag_name] = self.repository.lookup_foreign_revision_id(peeled)
+                try:
+                    updates[tag_name] = self.repository.lookup_foreign_revision_id(peeled)
+                except KeyError:
+                    trace.warning('%s does not point to a valid object',
+                                  tag_name)
             else:
-                source_revid = self.repository.lookup_foreign_revision_id(peeled)
                 try:
+                    source_revid = self.repository.lookup_foreign_revision_id(peeled)
                     target_revid = target_repo.lookup_foreign_revision_id(
                             target_repo._git.refs[ref_name])
                 except KeyError:

=== modified file 'breezy/git/interrepo.py'
--- breezy/git/interrepo.py	2018-11-10 17:52:47 +0000
+++ breezy/git/interrepo.py	2018-11-16 11:31:40 +0000
@@ -398,16 +398,20 @@
     def get_determine_wants_heads(self, wants, include_tags=False):
         wants = set(wants)
         def determine_wants(refs):
-            potential = set(wants)
+            unpeel_lookup = {}
+            for k, v in viewitems(refs):
+                if k.endswith(ANNOTATED_TAG_SUFFIX):
+                    unpeel_lookup[v] = refs[k[:-len(ANNOTATED_TAG_SUFFIX)]]
+            potential = set([unpeel_lookup.get(w, w) for w in wants])
             if include_tags:
-                for k, unpeeled in viewitems(refs):
+                for k, sha in viewitems(refs):
                     if k.endswith(ANNOTATED_TAG_SUFFIX):
                         continue
                     if not is_tag(k):
                         continue
-                    if unpeeled == ZERO_SHA:
+                    if sha == ZERO_SHA:
                         continue
-                    potential.add(unpeeled)
+                    potential.add(sha)
             return list(potential - self._target_has_shas(potential))
         return determine_wants
 
@@ -472,20 +476,6 @@
             potential.add(self.source.controldir.get_peeled(k) or v)
         return list(potential - self._target_has_shas(potential))
 
-    def get_determine_wants_heads(self, wants, include_tags=False):
-        wants = set(wants)
-        def determine_wants(refs):
-            potential = set(wants)
-            if include_tags:
-                for k, unpeeled in viewitems(refs):
-                    if not is_tag(k):
-                        continue
-                    if unpeeled == ZERO_SHA:
-                        continue
-                    potential.add(self.source.controldir.get_peeled(k) or unpeeled)
-            return list(potential - self._target_has_shas(potential))
-        return determine_wants
-
     def _warn_slow(self):
         if not config.GlobalConfig().suppress_warning('slow_intervcs_push'):
             trace.warning(

=== modified file 'breezy/git/remote.py'
--- breezy/git/remote.py	2018-11-16 10:50:21 +0000
+++ breezy/git/remote.py	2018-11-16 11:31:40 +0000
@@ -925,7 +925,9 @@
 
     def _synchronize_history(self, destination, revision_id):
         """See Branch._synchronize_history()."""
-        destination.generate_revision_history(self.last_revision())
+        if revision_id is None:
+            revision_id = self.last_revision()
+        destination.generate_revision_history(revision_id)
 
     def _get_parent_location(self):
         return None

=== modified file 'breezy/git/tests/test_remote.py'
--- breezy/git/tests/test_remote.py	2018-11-16 10:50:21 +0000
+++ breezy/git/tests/test_remote.py	2018-11-16 11:31:40 +0000
@@ -251,6 +251,38 @@
                 {'blah': default_mapping.revision_id_foreign_to_bzr(c2)},
                 local_branch.tags.get_tag_dict())
 
+    def test_sprout_with_annotated_tag_unreferenced(self):
+        c1 = self.remote_real.do_commit(
+                message=b'message',
+                committer=b'committer <committer@example.com>',
+                author=b'author <author@example.com>')
+        c2 = self.remote_real.do_commit(
+                message=b'another commit',
+                committer=b'committer <committer@example.com>',
+                author=b'author <author@example.com>')
+        porcelain.tag_create(
+                self.remote_real,
+                tag=b"blah",
+                author=b'author <author@example.com>',
+                objectish=c1,
+                tag_time=int(time.time()),
+                tag_timezone=0,
+                annotated=True,
+                message=b"Annotated tag")
+
+        remote = ControlDir.open(self.remote_url)
+        self.make_controldir('local', format=self._to_format)
+        local = remote.sprout(
+                'local',
+                revision_id=default_mapping.revision_id_foreign_to_bzr(c1))
+        local_branch = local.open_branch()
+        self.assertEqual(
+                default_mapping.revision_id_foreign_to_bzr(c1),
+                local_branch.last_revision())
+        self.assertEqual(
+                {'blah': default_mapping.revision_id_foreign_to_bzr(c1)},
+                local_branch.tags.get_tag_dict())
+
 
 class FetchFromRemoteToBzrTests(FetchFromRemoteTestBase,TestCaseWithTransport):
 

